%This program is set for interpreting data from HSCIP (.bin) files from Edgerton lab at UCLA
%Operations
    %reads analog channels, detects EMG envelope by TKEO Transform,
    %BANDPASS-BUTTERWORTH filters the EMG singal, calculates mean & standard deviation of
    %signal and voltage threshold (+2 SD), remove EMG signal that occured
    %during active periods (i.e. when rat was moving or grooming during MEP
    %data collection) based on stimulation pulses, makes all filtered
    %stimulation (< .2SD) to generate new stimulation signal
%Alignment
    %aligns motor evoked potentials (MEPs) for each forelimb muscle
    %(1-Deltoid, 2-Biceps, 3-Pronator, 4-Flexor and 5-Extensor); calculates average
    %MEP responses
%Numerical Calculations -> Tab (.txt) file
    %peak-to-peak amplitudes of early and late MEP responses
    %latency of MEP responses
    %area-under-curve of early and late MEP responses
%Graphs
    %Plot 1: Extract raw data from .bin file + detect stimuli
        %Discarded
    %Plot 2: 100ms-(A) Raw + mean MEP traces, (B) Rectified mean MEP trace
        %Saved,per current & configuration
    %Plot 3: 30ms-(A)Raw individual traces  
        %Saved,per current & configuration
%Update Date: 23 April 2018
%Contact: @ benita: bjin@ucla.edu
%================================================================================================
function U01_MEP_TKEOtransform_final()

% clear ram and close all the figures
close all

%VARIABLES
%Plot1
THR_MUL = 0.2;      % fine tuned for all files
STIM_CH = 6;        % stimulation channel
OFF = 10;           % off first 1 ms plotting to avoid stimulus artifact
%Plot2
POST_WINDOW = 100;   % plotting time window (ms)
MEP_WINDOW = 30;    % MEP time window for early response (ms)
PRE_WINDOW = 10;    % plotting time window before stim (ms)
SLOPE = 0.5;        % falling edge (slope) of stimulation pulse
ALL_ymax = 2;       % set bound limits for plot 2A
ALL_ymin = ALL_ymax * -1;
AVG_ymax = 1;       % set bound limits for plot 2B
%Plot3
POST3_MEP = 300;    % calculations for after pulse (10^-4 sec)
PRE3_MEP = 100;     % calculations for before pulse (10^-4 sec)
PRE_MEP = 10;       % plotting time window before pulse (ms)
POST_MEP = 30;      % plotting time window after pulse (ms)


%Treatment variable (for excel file)
%MUST UPDATE EACH TIME
Treatment = 'sham';

% sel_dir = ['G:\A\Documents\~Desktop_backup on 6-6-2017'...
%     '\4. Lab\~Transfer\Part3_2015_B animals\sMEP\Intact\Group1\'];
% sel_dir = 'G:\Edgerton\B animals_sMEP files\W06\';
mep_flag = false;
iemg = struct();
pk2pk = struct();

% sel_dir = 'G:\Edgerton\B animals_sMEP files\W06\';
% sel_dirs = {'.\Data\B_animals_sMEP_files\Intact', ...
%     '.\Data\B_animals_sMEP_files\W01',...
%     '.\Data\B_animals_sMEP_files\W06'};
sel_dirs = {'.\Data\B_animals_sMEP_files\Intact'};

hist_maxs = [];
hist_mins = [];
hist_struct = struct();
baseline_struct = struct();
b_index = 1;

nmep_lim = 75;

for nsel = 1 : length(sel_dirs)
    sel_dir = cell2mat(sel_dirs(nsel));
    addpath(genpath(sel_dir));
    file_list = dir(sel_dir);
    flist = {file_list.name};
    flist = flist(contains(flist,'Group'));
    ngroups = length(flist);

    if contains(lower(sel_dir), 'intact')
        base_flag = true;
        base = struct();
    else
        base_flag = false;
    end
    index = 1;

    for g = 1 : ngroups
        % Start a group for loop here maybe
        gname = cell2mat(flist(g));
        gnum  = str2num(gname(end));
        g_dir = [sel_dir '\' gname];

        mkdir([g_dir '\Figures']);
        mkdir([g_dir '\Data']);
        Save_filepath = [g_dir '\Figures\'];
        mep_filepath = [g_dir '\Data\'];

        files = dir([g_dir '\*.bin']);

        % batch process all the files from a folder
        % files = dir([g_dir '\*.bin']); % get all file info of the folder
        % Save_filepath = ['F:\A\Documents\~Desktop_backup on 6-6-2017\...
            % 4. Lab\Data Analyses\U01\2. Buspirone or Fluoxetine_2015\...
            % sMEP\Matlab-rep traces for figures\Deltoid\'];
        %note: add "\" at the end of filepath name 

        %LOOP to process all files
        for n_file = 1:length(files)

            % Open a bin file
            chosenfile = files(n_file).name;
            % uigetfile('*.*','Select the M-file');

            % define filename
            titles = chosenfile(25:end-4);
            heading = chosenfile(1:end-4);

            %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            %pull information from filename
            temp = find(heading(2:end) == 'W') + 1;
            Week = heading(temp+2);
            AnNum = heading(temp+4:temp+6);

            temp = find(heading == '+');
            Anode = heading(temp-2:temp-1);

            temp = find(heading == '-');
            Cathode = heading(temp-2:temp-1);

            temp = regexpi(heading, 'uA');
            Current = heading(temp-3:temp-1);
            %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            % Load EMG into data_array
            EMG_header = load_emg_header(chosenfile, 1);
            data_array = load_emg_data(chosenfile, ...
                EMG_header,EMG_header.data_position,1,inf,1);
            filtdata = zeros(length(data_array), 5);
            new_stim = zeros(length(data_array), 5);

            % system varibales
            keepgoing = false;
            MPstart = 1;
            stimamp = 0;
            counter = [];
            sampling = 10000; %kHz
            Niquest = sampling/2;
            trials = 0;
            %=========================================================================
            %ZEROS
            % average MEPs
            mean_MEP1 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            mean_MEP2 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            mean_MEP3 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            mean_MEP4 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            mean_MEP5 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            % rectified MEPs
            rec_MEP1 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            rec_MEP2 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            rec_MEP3 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            rec_MEP4 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            rec_MEP5 = zeros(PRE_WINDOW+POST_WINDOW+1,1);
            % rectified MEPs for baseline
            rec_burst1 = zeros(PRE_WINDOW+1,1);
            rec_burst2 = zeros(PRE_WINDOW+1,1);
            rec_burst3 = zeros(PRE_WINDOW+1,1);
            rec_burst4 = zeros(PRE_WINDOW+1,1);
            rec_burst5 = zeros(PRE_WINDOW+1,1);
            % response variables (peak to peak amplitude)
            early_amp1 = 0; late_amp1 = 0;
            early_amp2 = 0; late_amp2 = 0;
            early_amp3 = 0; late_amp3 = 0;
            early_amp4 = 0; late_amp4 = 0;
            early_amp5 = 0; late_amp5 = 0;
            % latency variables
            latency_early1 = 0;
            latency_early2 = 0;
            latency_early3 = 0;
            latency_early4 = 0;
            latency_early5 = 0;
            % area under rectified MEP responses
            auc_early1 = 0; auc_late1 = 0;
            auc_early2 = 0; auc_late2 = 0;
            auc_early3 = 0; auc_late3 = 0;
            auc_early4 = 0; auc_late4 = 0;
            auc_early5 = 0; auc_late5 = 0;
            %=========================================================================
            %=========================================================================
            % get the computer screen size
            scrsz = get(0,'ScreenSize');
            nrows = 6; % no of row
            ncols = 1; % no of column
            % calculate subplot position by calling subfigrid funtion
            pos = subfigrid(nrows,ncols,...
                  [0.07 0.01 0.02 0.13],[0.85 0.85]); % [left Right Top Bottom]

            % creating time vector
            tend = length(data_array(:,STIM_CH))/sampling;
            tvec = 0:1/sampling:tend;
            tvec(1) = [];   % sampling start from time zero 

            %=========================================================================
            %-----------------------------------------------------------------------
            %CH1 - Deltoid
            %BUTTERWORTH (BANDPASS) processing
            [b,a] = butter(2, [30 1000]/Niquest, 'bandpass');
            % filter channel 1 data
            filtdata_1 = filter(b,a,data_array(:,1));
            filtdata(:,1) = filtdata_1;
            % TKEO operation on channel 1
            x1 = data_array(:,1);   
            sqx1=x1(2:length(x1)-1).^2;
            minusx1=x1(1:length(x1)-2);
            plusx1=x1(3:length(x1));
            y1= sqx1 - (minusx1.*plusx1);
            y1 = [y1(1); y1; y1(length(x1)-2)];
            % Low-pass filter and rectification
            [b,a] = butter(2, 50/Niquest, 'low');
            % filter channel 2 data
            y1 = filter(b,a,y1);
            y1 = abs(y1);
            % Thresholding
            thr1 = mean(y1) + THR_MUL*std(y1);
            thy1 = zeros(length(x1),1)+thr1;
            % Masking
            msk1 = y1;
            msk1(msk1>thy1) = 1;  %active period
            msk1(msk1<=thy1) = 0; %inactive period
            % New stimulation
            new_stim_1 = times(not(msk1),data_array(:,STIM_CH));
            new_stim(:,1) = new_stim_1;
            %PLOT1
            h1 = figure('name',titles,'Position',[1 scrsz(2) scrsz(3) scrsz(4)]);

            % plot figure
            subplot('position',pos(1,:,1)); 
            plot(tvec, filtdata_1, 'color', 'black');
            hold on;
            plot(tvec, msk1, 'color', 'cyan');
            plot(tvec, y1*5,'color','green');
            plot(tvec, thy1*5, 'color', 'blue');
            plot(tvec, new_stim_1, 'LineWidth', 1.5, 'color', 'red');
            xlim([1 length(tvec)/sampling]); ylim([-1 1]);
            set(gca,'fontsize',16); ylabel('Ch 1','fontsize',16);
            %-----------------------------------------------------------------------
            %CH2 - Biceps
            %BUTTERWORTH (BANDPASS) processing
            [b,a] = butter(2, [30 1000]/Niquest, 'bandpass');
            % filter channe 2 data
            filtdata_2 = filter(b,a,data_array(:,2));
            filtdata(:,2) = filtdata_2;
            % TKEO operation on channel 2
            x2 = data_array(:,2);
            sqx2=x2(2:length(x2)-1).^2;
            minusx2=x2(1:length(x2)-2);
            plusx2=x2(3:length(x2));
            y2= sqx2 - (minusx2.*plusx2);
            y2 = [y2(1); y2; y2(length(x1)-2)];
            % Low-pass filter and rectification
            [b,a] = butter(2, 50/Niquest, 'low');
            % filter channe 2 data
            y2 = filter(b,a,y2);
            y2 = abs(y2);
            % Thresholding
            thr2 = mean(y2) + THR_MUL*std(y2);
            thy2 = zeros(length(x2),1)+thr2;
            % Masking
            msk2 = y2;
            msk2(msk2>thy2) = 1;  %active period
            msk2(msk2<=thy2) = 0; %inactive period
            % New stimulation
            new_stim_2 = times(not(msk2),data_array(:,STIM_CH));
            new_stim(:,2) = new_stim_2;
            % plot figure
            subplot('position',pos(2,:,1)); 
            plot(tvec, filtdata_2, 'color', 'black');
            hold on;
            plot(tvec, msk2, 'color', 'cyan');
            plot(tvec, y2*5,'color','green');
            plot(tvec, thy2*5, 'color', 'blue');
            plot(tvec, new_stim_2, 'LineWidth', 1.5, 'color', 'red');
            xlim([1 length(tvec)/sampling]); ylim([-1 1]);
            set(gca,'fontsize',16); ylabel('Ch 2','fontsize',16);
            %-----------------------------------------------------------------------
            % CH3 - Pronator
            %BUTTERWORTH (BANDPASS) processing
            [b,a] = butter(2, [30 1000]/Niquest, 'bandpass');
            % filter channel 3 data
            filtdata_3 = filter(b,a,data_array(:,3));
            filtdata(:,3) = filtdata_3;
            % TKEO operation on channel 3
            x3 = data_array(:,3);
            sqx3=x3(2:length(x3)-1).^2;
            minusx3=x3(1:length(x3)-2);
            plusx3=x3(3:length(x3));
            y3= sqx3 - (minusx3.*plusx3);
            y3 = [y3(1); y3; y3(length(x3)-2)];
            % Low-pass filter and rectification
            [b,a] = butter(2, 50/Niquest, 'low');
            % filter channe 2 data
            y3 = filter(b,a,y3);
            y3 = abs(y3);
            % Thresholding
            thr3 = mean(y3) + THR_MUL*std(y3);
            thy3 = zeros(length(x3),1)+thr3;
            % Masking
            msk3 = y3;
            msk3(msk3>thy3) = 1;  %active period
            msk3(msk3<=thy3) = 0; %inactive period
            % New stimulation
            new_stim_3 = times(not(msk3),data_array(:,STIM_CH));
            new_stim(:,3) = new_stim_3;
            % plot figure
            subplot('position',pos(3,:,1)); 
            plot(tvec, filtdata_3, 'color', 'black');
            hold on;
            plot(tvec, msk3, 'color', 'cyan');
            plot(tvec, y3*5,'color','green');
            plot(tvec, thy3*5, 'color', 'blue');
            plot(tvec, new_stim_3, 'LineWidth', 1.5, 'color', 'red');
            xlim([1 length(tvec)/sampling]); ylim([-1 1]);
            set(gca,'fontsize',16); ylabel('Ch 3','fontsize',16);
            %-----------------------------------------------------------------------
            % CH4 - Flexor
            %BUTTERWORTH (BANDPASS) processing
            [b,a] = butter(2, [30 1000]/Niquest, 'bandpass');
            % filter channe 4 data
            filtdata_4 = filter(b,a,data_array(:,4));
            filtdata(:,4) = filtdata_4;
            % TKEO operation on channel 4
            x4 = data_array(:,4);
            sqx4=x4(2:length(x4)-1).^2;
            minusx4=x4(1:length(x4)-2);
            plusx4=x4(3:length(x4));
            y4= sqx4 - (minusx4.*plusx4);
            y4 = [y4(1); y4; y4(length(x4)-2)];
            % Low-pass filter and rectification
            [b,a] = butter(2, 50/Niquest, 'low');
            % filter channe 2 data
            y4 = filter(b,a,y4);
            y4 = abs(y4);
            % Thresholding
            thr4 = mean(y4) + THR_MUL*std(y4);
            thy4 = zeros(length(x4),1)+thr4;
            % Masking
            msk4 = y4;
            msk4(msk4>thy4) = 1;  %active period
            msk4(msk4<=thy4) = 0; %inactive period
            % New stimulation
            new_stim_4 = times(not(msk4),data_array(:,STIM_CH));
            new_stim(:,4) = new_stim_4;
                % plot figure
            subplot('position',pos(4,:,1)); 
            plot(tvec, filtdata_4, 'color', 'black');
            hold on;
            plot(tvec, msk4, 'color', 'cyan');
            plot(tvec, y4*5,'color','green');
            plot(tvec, thy4*5, 'color', 'blue');
            plot(new_stim_4, 'LineWidth', 1.5, 'color', 'red');
            xlim([1 length(tvec)/sampling]); ylim([-1 1]);
            set(gca,'fontsize',16); ylabel('Ch 4','fontsize',16);
            %-----------------------------------------------------------------------
            % CH5 - Extensor
            %BUTTERWORTH (BANDPASS) processing
            [b,a] = butter(2, [30 1000]/Niquest, 'bandpass');
            % filter channel 5 data
            filtdata_5 = filter(b,a,data_array(:,5));
            filtdata(:,5) = filtdata_5;
            % TKEO operation on channel 5
            x5 = data_array(:,5);
            sqx5=x5(2:length(x5)-1).^2;
            minusx5=x5(1:length(x5)-2);
            plusx5=x5(3:length(x5));
            y5= sqx5 - (minusx5.*plusx5);
            y5 = [y5(1); y5; y5(length(x5)-2)];
            % Low-pass filter and rectification
            [b,a] = butter(2, 50/Niquest, 'low');
            % filter channe 2 data
            y5 = filter(b,a,y5);
            y5 = abs(y5);
            % Thresholding
            thr5 = mean(y5) + THR_MUL*std(y5);
            thy5 = zeros(length(x5),1)+thr5;
            % Masking
            msk5 = y5;
            msk5(msk5>thy5) = 1;  %active period
            msk5(msk5<=thy5) = 0; %inactive period
            % New stimulation
            new_stim_5 = times(not(msk5),data_array(:,STIM_CH));
            new_stim(:,5) = new_stim_5;
            % plot figure
            subplot('position',pos(5,:,1)); 
            plot(tvec, filtdata_5, 'color', 'black');
            hold on;
            plot(tvec, msk5, 'color', 'cyan');
            plot(tvec, y5*5,'color','green');
            plot(tvec, thy5*5, 'color', 'blue');
            plot(tvec, new_stim_5, 'LineWidth', 1.5, 'color', 'red');
            xlim([1 length(tvec)/sampling]); ylim([-1 1]);
            set(gca,'fontsize',16); ylabel('Ch 5','fontsize',16);
            %=========================================================================
            % final stimulation signal (masked all above new stimulations)
            final_stim = and(round(new_stim_1), round(new_stim_2));
            final_stim = and(final_stim, round(new_stim_3));
            final_stim = and(final_stim, round(new_stim_4));
            final_stim = and(final_stim, round(new_stim_5));

            session = length(final_stim);

            subplot('position',pos(6,:,1)); plot(tvec, final_stim, 'LineWidth', 1.5, 'color', 'blue');
            xlim([1 session/sampling]); ylim([-0.5 1.5]);
            set(gca,'fontsize',16); xlabel('Sec','fontsize',16); ylabel('Ch 6','fontsize',16);
            % save final_stim_ch;

            %=========================================================================
            % MEP responses here
            %=========================================================================
            fname = chosenfile(1:end-4);
            split_fname = strsplit(fname,'_');

            b_name = [cell2mat(split_fname(3)) '\'];
            dir_list = dir([mep_filepath b_name]);
            if length(dir_list) < 1
                mkdir([mep_filepath b_name]);
            end
            if mep_flag
                for c = 1 : 5
                    s1 = sprintf('ch%d',c);
                    csv_fname = sprintf('%s\\%s\\%s_%s',mep_filepath,b_name,...
                        fname, s1);

                    [iemg_temp , pk2pk_temp] = mepresponse(filtdata(:,c), ...
                        new_stim(:,c), sampling, c);

                    iemg(index, 1).Week = Week;
                    iemg(index, 1).AnNum = AnNum;
                    iemg(index, 1).Anode = Anode;
                    iemg(index, 1).Cathode = Cathode;
                    iemg(index, 1).Current = Current;
                    iemg(index, 1).Group = gnum;
                    iemg(index, 1).Channel = c;
                    iemg(index, 1).MiddleResponses = iemg_temp.mr;
                    iemg(index, 1).LateResponses = iemg_temp.lr;

                    pk2pk(index, 1).Week = Week;
                    pk2pk(index, 1).AnNum = AnNum;
                    pk2pk(index, 1).Anode = Anode;
                    pk2pk(index, 1).Cathode = Cathode;
                    pk2pk(index, 1).Current = Current;
                    pk2pk(index, 1).Group = gnum;
                    pk2pk(index, 1).Channel = c;
                    pk2pk(index, 1).MRMeans = pk2pk_temp.mr_means;
                    pk2pk(index, 1).LRMeans = pk2pk_temp.lr_means;
                    pk2pk(index, 1).MRPeak2Peak = pk2pk_temp.mr;
                    pk2pk(index, 1).LRPeak2Peak = pk2pk_temp.lr;
                    pk2pk(index, 1).MRNumPeaks = pk2pk_temp.mr_num_pks;
                    pk2pk(index, 1).LRNumPeaks = pk2pk_temp.lr_num_pks;
                    
                    index = index + 1;
                    
                end
            end


            %=========================================================================
            %=========================================================================
            %PLOT2
            % Find stimpulse start
            threshold = SLOPE;
            offsetData = [final_stim(2:end);  NaN];
            risingEdge = find(final_stim < threshold & offsetData > threshold);
            fallingEdge = find(final_stim > threshold & offsetData < threshold);
            % figure; plot(final_stim);
            % time = 1:1:length(final_stim);
            % hold on;
            % plot(time(risingEdge), threshold, 'rx');
            % plot(time(fallingEdge), threshold, 'go');
            counter = risingEdge';

            % print the number of trials taken
            trials = length(counter)-1;  

            % get the computer screen size
            scrsz = get(0,'ScreenSize');
            nrows = 2; % no of row
            ncols = 5; % no of column
            % calculate subplot position by calling subfigrid funtion
            pos = subfigrid(nrows,ncols,...
                  [0.06 0.01 0.06 0.08],[0.84 0.88]); % [left Right Top Bottom]

            % h2 = figure('name',heading,'Position',[1 scrsz(2) scrsz(3) scrsz(4)]);
            MEP_time = (-PRE_WINDOW/sampling:1/sampling:POST_WINDOW/sampling)*10000'; 
            %-----------------------------------------------------------------------
            %CH1 - Deltoid
            subplot('position',pos(1,:,1));
            for j=1:1:trials
                k = counter(:,j);
                if k < PRE_WINDOW
                    k = PRE_WINDOW + 1;
                elseif k+POST_WINDOW > length(filtdata_1)
                    k = length(filtdata_1) - POST_WINDOW;
                end
                plot(MEP_time, filtdata_1((k-PRE_WINDOW):(k+POST_WINDOW)));
                mean_MEP1 = mean_MEP1 + filtdata_1((k-PRE_WINDOW):(k+POST_WINDOW));
                rec_MEP1 = rec_MEP1 + abs(filtdata_1((k-PRE_WINDOW):(k+POST_WINDOW)));
                if k < PRE3_MEP
                    k = PRE3_MEP + 1;
                elseif k+POST3_MEP > length(filtdata_1)
                    k = length(filtdata_1) - POST3_MEP;
                end
                out_MEP1(:,j) = filtdata_1((k-PRE3_MEP):(k+POST3_MEP));
                hold all
            end
            mean_MEP1 = mean_MEP1/j;
            plot([0, 0], [ALL_ymin ALL_ymax], 'r')
            ylim([ALL_ymin ALL_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Deltoid'], 'interpreter', 'none', 'fontsize',14); ylabel('mv','fontsize',16);

            rec_MEP1 = rec_MEP1/j;
            subplot('position',pos(2,:,1)); 
            plot(MEP_time, rec_MEP1, 'LineWidth', 1.5, 'color', 'blue');
            hold on
            plot([0, 0], [ALL_ymin ALL_ymax], 'r') %adds line to mark stimulation pulse onset
            hold off
            ylim([0 AVG_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title('Deltoid', 'interpreter', 'none', 'fontsize',14); xlabel('ms','fontsize',18); ylabel('|mv|','fontsize',16);

            %below calculations are from data at 0-30msec after stimulation pulse onset
            %average peak-to-peak responses
            early_amp1 = max(mean_MEP1(OFF+1:MEP_WINDOW)) - min(mean_MEP1(OFF+1:MEP_WINDOW));
            late_amp1 = max(mean_MEP1(MEP_WINDOW+1:POST_WINDOW)) - min(mean_MEP1(MEP_WINDOW+1:POST_WINDOW));
            % latency calculation
            [Max1 Imax1] = max(abs(mean_MEP1(OFF+1:MEP_WINDOW)));
            latency_early1 = Imax1/10; %latency in ms
            %area under curve
            auc_early1 = trapz(rec_MEP1(OFF+1:MEP_WINDOW));
            auc_late1 = trapz(rec_MEP1(MEP_WINDOW+1:POST_WINDOW));
            %-----------------------------------------------------------------------
            %CH2 - Biceps
            subplot('position',pos(1,:,2));
            for j=1:1:trials
                k = counter(:,j);
                if k < PRE_WINDOW
                    k = PRE_WINDOW + 1;
                elseif k+POST_WINDOW > length(filtdata_2)
                    k = length(filtdata_2) - POST_WINDOW;
                end
                plot(MEP_time, filtdata_2((k-PRE_WINDOW):(k+POST_WINDOW)));
                mean_MEP2 = mean_MEP2 + filtdata_2((k-PRE_WINDOW):(k+POST_WINDOW));
                rec_MEP2 = rec_MEP2 + abs(filtdata_2((k-PRE_WINDOW):(k+POST_WINDOW)));
                if k < PRE3_MEP
                    k = PRE3_MEP + 1;
                elseif k+POST3_MEP > length(filtdata_2)
                    k = length(filtdata_2) - POST3_MEP;
                end
                out_MEP2(:,j) = filtdata_2((k-PRE3_MEP):(k+POST3_MEP));
                hold all
            end
            mean_MEP2 = mean_MEP2/j;
            plot(MEP_time, mean_MEP2, 'LineWidth', 2.5, 'color', 'black');
            plot([0, 0], [ALL_ymin ALL_ymax], 'r');
            ylim([ALL_ymin ALL_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Biceps'], 'interpreter', 'none', 'fontsize',14);

            rec_MEP2 = rec_MEP2/j;
            subplot('position',pos(2,:,2)); 
            plot(MEP_time, rec_MEP2, 'LineWidth', 1.5, 'color', 'blue');
            hold on
            plot([0,0], [ALL_ymin ALL_ymax], 'r')
            hold off
            ylim([0 AVG_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Biceps'], 'interpreter', 'none', 'fontsize',14); xlabel('ms','fontsize',16);

            %average peak-to-peak responses
            early_amp2 = max(mean_MEP2(OFF+1:MEP_WINDOW)) - min(mean_MEP2(OFF+1:MEP_WINDOW));
            late_amp2 = max(mean_MEP2(MEP_WINDOW+1:POST_WINDOW)) - min(mean_MEP2(MEP_WINDOW+1:POST_WINDOW));
            % latency calculation
            [Max2 Imax2] = max(abs(mean_MEP2(OFF+1:MEP_WINDOW)));
            latency_early2 = Imax2/10; %latency in ms
            % area under curve
            auc_early2 = trapz(rec_MEP2(OFF+1:MEP_WINDOW));
            auc_late2 = trapz(rec_MEP2(MEP_WINDOW+1:POST_WINDOW));
            %-----------------------------------------------------------------------
            %Ch3 - Pronator
            subplot('position',pos(1,:,3));
            for j=1:1:trials
                k = counter(:,j);
                if k < PRE_WINDOW
                    k = PRE_WINDOW + 1;
                elseif k+POST_WINDOW > length(filtdata_3)
                    k = length(filtdata_1) - POST_WINDOW;
                end
                plot(MEP_time, filtdata_3((k-PRE_WINDOW):(k+POST_WINDOW)));
                mean_MEP3 = mean_MEP3 + filtdata_3((k-PRE_WINDOW):(k+POST_WINDOW));
                rec_MEP3 = rec_MEP3 + abs(filtdata_3((k-PRE_WINDOW):(k+POST_WINDOW)));
                if k < PRE3_MEP
                    k = PRE3_MEP + 1;
                elseif k+POST3_MEP > length(filtdata_3)
                    k = length(filtdata_3) - POST3_MEP;
                end
                out_MEP3(:,j) = filtdata_3((k-PRE3_MEP):(k+POST3_MEP));
                hold all;
            end
            mean_MEP3 = mean_MEP3/j;
            plot(MEP_time, mean_MEP3, 'LineWidth', 2.5, 'color', 'black');
            plot ([0,0], [ALL_ymin ALL_ymax], 'r')
            ylim([ALL_ymin ALL_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Pronator'], 'interpreter', 'none', 'fontsize',14);

            rec_MEP3 = rec_MEP3/j;
            subplot('position',pos(2,:,3)); 
            plot(MEP_time, rec_MEP3, 'LineWidth', 1.5, 'color', 'blue');
            hold on
            plot([0,0], [ALL_ymin ALL_ymax], 'r') 
            hold off
            ylim([0 AVG_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Pronator'], 'interpreter', 'none', 'fontsize',14); xlabel('ms','fontsize',16);

            %average peak-to-peak responses
            early_amp3 = max(mean_MEP3(OFF+1:MEP_WINDOW)) - min(mean_MEP3(OFF+1:MEP_WINDOW));
            late_amp3 = max(mean_MEP3(MEP_WINDOW+1:POST_WINDOW)) - min(mean_MEP3(MEP_WINDOW+1:POST_WINDOW));
            % latency calculation
            [Max3 Imax3] = max(abs(mean_MEP3(OFF+1:MEP_WINDOW)));
            latency_early3 = Imax3/10; %latency in ms
            % area under curve
            auc_early3 = trapz(rec_MEP3(OFF+1:MEP_WINDOW));
            auc_late3 = trapz(rec_MEP3(MEP_WINDOW+1:POST_WINDOW));
            %-----------------------------------------------------------------------
            %Ch4 - Flexor
            subplot('position',pos(1,:,4));
            for j=1:1:trials
                k = counter(:,j);
                if k < PRE_WINDOW
                    k = PRE_WINDOW + 1;
                elseif k+POST_WINDOW > length(filtdata_4)
                    k = length(filtdata_1) - POST_WINDOW;
                end
                plot(MEP_time, filtdata_4((k-PRE_WINDOW):(k+POST_WINDOW)));
                mean_MEP4 = mean_MEP4 + filtdata_4((k-PRE_WINDOW):(k+POST_WINDOW));
                rec_MEP4 = rec_MEP4 + abs(filtdata_4((k-PRE_WINDOW):(k+POST_WINDOW)));
                if k < PRE3_MEP
                    k = PRE3_MEP + 1;
                elseif k+POST3_MEP > length(filtdata_4)
                    k = length(filtdata_4) - POST3_MEP;
                end
                out_MEP4(:,j) = filtdata_4((k-PRE3_MEP):(k+POST3_MEP));
                hold all
            end
            mean_MEP4 = mean_MEP4/j;
            plot(MEP_time, mean_MEP4, 'LineWidth', 2.5, 'color', 'black');
            plot([0, 0], [ALL_ymin ALL_ymax], 'r')
            ylim([ALL_ymin ALL_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Flexor'], 'interpreter', 'none', 'fontsize',14);

            rec_MEP4 = rec_MEP4/j;
            subplot('position',pos(2,:,4)); 
            plot(MEP_time, rec_MEP4, 'LineWidth', 1.5, 'color', 'blue');
            hold on
            plot ([0,0], [ALL_ymin ALL_ymax], 'r')
            hold off
            ylim([0 AVG_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Flexor'], 'interpreter', 'none', 'fontsize',14); xlabel('ms','fontsize',16);

            %average peak-to-peak responses
            early_amp4 = max(mean_MEP4(OFF+1:MEP_WINDOW)) - min(mean_MEP4(OFF+1:MEP_WINDOW));
            late_amp4 = max(mean_MEP4(MEP_WINDOW+1:POST_WINDOW)) - min(mean_MEP4(MEP_WINDOW+1:POST_WINDOW));
            % latency calculation
            [Max4 Imax4] = max(abs(mean_MEP4(OFF+1:MEP_WINDOW)));
            latency_early4 = Imax4/10; %latency in ms
            % area under curve
            auc_early4 = trapz(rec_MEP4(OFF+1:MEP_WINDOW));
            auc_late4 = trapz(rec_MEP4(MEP_WINDOW+1:POST_WINDOW));
            %-----------------------------------------------------------------------
            %CH5 - Extensor
            subplot('position',pos(1,:,5));
            for j=1:1:trials
                k = counter(:,j);
                if k < PRE_WINDOW
                    k = PRE_WINDOW + 1;
                elseif k+POST_WINDOW > length(filtdata_5)
                    k = length(filtdata_1) - POST_WINDOW;
                end
                plot(MEP_time, filtdata_5((k-PRE_WINDOW):(k+POST_WINDOW)));
                mean_MEP5 = mean_MEP5 + filtdata_5((k-PRE_WINDOW):(k+POST_WINDOW));
                rec_MEP5 = rec_MEP5 + abs(filtdata_5((k-PRE_WINDOW):(k+POST_WINDOW)));
                if k < PRE3_MEP
                    k = PRE3_MEP + 1;
                elseif k+POST3_MEP > length(filtdata_5)
                    k = length(filtdata_5) - POST3_MEP;
                end
                out_MEP5(:,j) = filtdata_5((k-PRE3_MEP):(k+POST3_MEP));
                hold all
            end
            mean_MEP5 = mean_MEP5/j;
            plot(MEP_time, mean_MEP5, 'LineWidth', 2.5, 'color', 'black');
            plot([0, 0], [ALL_ymin ALL_ymax], 'r')
            ylim([ALL_ymin ALL_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Extensor'], 'interpreter', 'none', 'fontsize',14);

            rec_MEP5 = rec_MEP5/j;
            subplot('position',pos(2,:,5)); 
            plot(MEP_time, rec_MEP5, 'LineWidth', 1.5, 'color', 'blue');
            hold on
            plot ([0,0], [ALL_ymin ALL_ymax], 'r')
            hold off
            ylim([0 AVG_ymax]); 
            xlim([-PRE_WINDOW POST_WINDOW]);
            title(['Extensor'], 'interpreter', 'none', 'fontsize',14); xlabel('ms','fontsize',18);

            % average peak-to-peak responses
            early_amp5 = max(mean_MEP5(OFF+1:MEP_WINDOW)) - min(mean_MEP5(OFF+1:MEP_WINDOW));
            late_amp5 = max(mean_MEP5(MEP_WINDOW+1:POST_WINDOW)) - min(mean_MEP5(MEP_WINDOW+1:POST_WINDOW));
            % latency calculation
            [Max5 Imax5] = max(abs(mean_MEP5(OFF+1:MEP_WINDOW)));
            latency_early5 = Imax5/10; %latency in ms
            % area under curve
            auc_early5 = trapz(rec_MEP5(OFF+1:MEP_WINDOW));
            auc_late5 = trapz(rec_MEP5(MEP_WINDOW+1:POST_WINDOW));

            % %**************************************************************************
            % % figure saving (change to folder where you want to save)
            % %**************************************************************************
            % saveas(h2, [Save_filepath, heading, '.tif']);

            %=========================================================================
            %PLOT3
            scrsz = get(0,'ScreenSize');
            nrows = 1; % no of row
            ncols = 5; % no of column
            % calculate subplot position by calling subfigrid funtion
            pos = subfigrid(nrows,ncols,...
                  [0.06 0.01 0.06 0.08],[0.84 0.88]); % [left Right Top Bottom]

            h2 = figure('name',heading,'Position',[1 scrsz(2) scrsz(3) scrsz(4)]);
            MEP_time = (-PRE3_MEP/sampling:1/sampling:POST3_MEP/sampling)*1000';% not sure why *1000
            offset1 = repmat(1:75, size(out_MEP1,1), 1);
            offset2 = repmat(1:75, size(out_MEP2,1), 1);
            offset3 = repmat(1:75, size(out_MEP3,1), 1);
            offset4 = repmat(1:75, size(out_MEP4,1), 1);
            offset5 = repmat(1:75, size(out_MEP5,1), 1);

            if base_flag
                %{
                    chk_len = size(out_MEP1,2);
                    if chk_len < nmep_lim
                        lim = chk_len;
                    else
                        lim = nmep_lim;
                    end
                %}
                out_MEPs = zeros(size(out_MEP1,1), size(out_MEP1,2),5);
                out_MEPs(:,:,1) = out_MEP1(:,:);
                out_MEPs(:,:,2) = out_MEP2(:,:);
                out_MEPs(:,:,3) = out_MEP3(:,:);
                out_MEPs(:,:,4) = out_MEP4(:,:);
                out_MEPs(:,:,5) = out_MEP5(:,:);

            % get min from all recordings across each channel and move to
            % 2d array
                out_mins = squeeze(min(out_MEPs, [], 1));
                out_maxs = squeeze(max(out_MEPs, [], 1));
                abs_maxs = squeeze(max(abs(out_MEPs),[],1));

                split_fname = split(chosenfile(1 : end-4),'_');
                week = split_fname{2};
                bnum = split_fname{3};
                electrode = [split_fname{6} '_' split_fname{7}];
                current = split_fname{end};

                baseline_struct.(week)(b_index).AnNum = AnNum;
                baseline_struct.(week)(b_index).Group = gnum;
                baseline_struct.(week)(b_index).Anode = Anode;
                baseline_struct.(week)(b_index).Cathode = Cathode;
                baseline_struct.(week)(b_index).Current = Current;
                baseline_struct.(week)(b_index).maxs = out_maxs;
                baseline_struct.(week)(b_index).mins = out_mins;
                baseline_struct.(week)(b_index).abs_max = abs_maxs;

                hist_mins = [hist_mins ; out_mins];
                hist_maxs = [hist_maxs ; out_maxs];
                
                b_index = b_index + 1;
            end

            %----------------------------------------------------------------------- 
            %CH1 - Deltoid
            if size(out_MEP1,2) < 75
                t = 75 - size(out_MEP1,2);
                out_MEP1 = [out_MEP1, NaN(size(out_MEP1,1), t)];
            elseif size(out_MEP1,2) > 75
                out_MEP1 = out_MEP1(:, 1:75);
            end

            subplot('position',pos(1,:,1));
            plot(MEP_time, out_MEP1(:, 1:75)+offset1, 'color', 'black');
            hold on
            plot ([0,0], [19,61], 'r');
            plot ([10,10], [19,61], 'b');
            hold off
            ylim([19, 61]);
            xlim([-PRE_MEP POST_MEP]); 
            title(['Deltoid'], 'interpreter', 'none', 'fontsize',14);

            %**************************************************************************
            % figure saving (change to folder where you want to save)
            %**************************************************************************
            % saveas(gcf, [Save_filepath, heading, '.bmp']);

            %**************************************************************************
            % data saving (change to folder where you want to save)
            %**************************************************************************
            % close all the figures
            close all;
            % create and save file containing 
            %n--mean sMEP--p-p amplitute of ealy and late sMEP--latency of early sMEP peak, 
            %mean recMEP--AUC of early and late recMEP
            %generate excel file in same location as Save_filepath with numerical data
            txt_output = {heading, AnNum, Week, Treatment, Current, Anode, Cathode};
            num_output = [trials, early_amp1, early_amp2, early_amp3,...
                early_amp4, early_amp5, late_amp1, late_amp2, late_amp3, ...
                late_amp4, late_amp5, latency_early1, latency_early2,...
                latency_early3, latency_early4, latency_early5, auc_early1,...
                auc_early2, auc_early3, auc_early4, auc_early5, auc_late1,...
                auc_late2, auc_late3, auc_late4, auc_late5];

            if exist([Save_filepath, 'MEPvalues.xlsx'], 'file') == 0
                chosenfile = [Save_filepath, 'MEPvalues.xlsx'];
                sheet = 1;
                x1cell = 'A2';
                x2cell = 'H2';
                labels ={'heading', 'AnNum', 'Week', 'Treatment', 'Current', 'Anode','Cathode','Trials','Deltoid early P-P','Biceps eP-P','Pronator eP-P','Flexor eP-P','Extensor eP-P','Deltoid late P-P','Biceps lP-P','Pronator lP-P','Flexor lP-P','Extensor 1P-P','D latency','B latency','P latency','F latency','E latency','D eAUC','B eAUC','P eAUC','F eAUC','E eAUC','D lAUC','B lAUC','P lAUC','F lAUC','E lAUC'};
                xlswrite(chosenfile,labels);
                [num,txt,raw]=xlsread(chosenfile);
                xlswrite(chosenfile,txt_output,sheet, x1cell);
                xlswrite(chosenfile,num_output,sheet, x2cell);
                fprintf([heading '\n']);
            elseif exist([Save_filepath, 'MEPvalues.xlsx'],'file') > 0
                 chosenfile = [Save_filepath, 'MEPvalues.xlsx'];
                x1cell = 'A2';
                x2cell = 'H2';
                [num,txt,raw]=xlsread(chosenfile);
                xlswrite(chosenfile,vertcat(raw(2:end,1:7),txt_output),1,x1cell);
                xlswrite(chosenfile,vertcat(num(1:end,6:end),num_output),1,x2cell);
                fprintf([heading '\n']);
            end

            fclose('all');
            %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            clear out_MEP1 out_MEP2 out_MEP3 out_MEP4 out_MEP5;

        end
        if base_flag
            hist_struct.(gname).hist_mins = hist_mins;
            hist_struct.(gname).hist_maxs = hist_maxs;
        end
        hist_mins = [];
        hist_maxs = [];

    end
    save('hist_struct.mat','hist_struct','-v7.3');
    save('baseline_struct.mat','baseline_struct','-v7.3');
    if mep_flag
        struct2csv(iemg , 'iemg.csv');
        struct2csv(pk2pk, 'pks.csv');
    end
end
end
