function [pk2pk, npks] = calculatePk2Pk(data, minProminence)


[~, ncols] = size(data);
pk2pk = zeros(1, ncols);
npks = zeros(1, ncols);

% [~ , thold_cfg, ~] = mep_config();
% minPkDist   = thold_cfg.minPkDist;

pk_mask = islocalmax(data, 'MinProminence', minProminence);
vly_mask = islocalmin(data, 'MinProminence', minProminence);

t = 1 : size(data,1);

for c = 1 : ncols
    pk2pk_ = peak2peak(pk_mask(:,c), vly_mask(:,c), data(:,c), t);
    plot(t, data(:,c), 'k-', ...
        t(pk_mask(:,c)), data(pk_mask(:,c),c), 'b*',...
        t(vly_mask(:,c)), data(vly_mask(:,c),c), 'r*');
    temp = pk2pk_( ~isnan(pk2pk_) );
    if ~isempty(temp)
        pk2pk(c) = max(temp);
    end
    npks(c) = length(pk2pk_);
end

end

function pk2pk = peak2peak(lmax_mask, lmin_mask, y, sect)
    % This function cycles through each peak and looks forwards for any
    % valleys, if there are none in the defined window, it looks backwards.
    num_pks = 0;

    % Define the peak indexes
    pk_mask = lmax_mask(sect);
    pk_inds = sect(pk_mask);

    % Define the peaks
    y_sect = y(sect);
    pks = y_sect(pk_mask);
    pk2pk = nan;

    % Loop through each peak
    for i = 1 : length(pk_inds)
        % Select first peak
        pk = pks(i);
        pk_ind = pk_inds(i);

        % Look at the forward indexes
        fwd_inds = pk_ind : sect(end);

        % Check for troughs
        temp = lmin_mask(fwd_inds);
        temp = fwd_inds(temp);

        % If there are none, then there is no peak to peak value for this
        % peak, move on to the next one
        if isempty(temp)
            num_pks = num_pks + 1;
            pk2pk(num_pks) = nan;
            continue;
        end

        % If there is a trough then select the first found trough, calculate
        % the peak to peak value, then set the mask index to be false so we do
        % not count it again.
        vly_ind = temp(1);
        vly = y(vly_ind);
        num_pks = num_pks + 1;
        pk2pk(num_pks) = pk - vly;

        lmin_mask(vly_ind) = false;
    end
end
