function main()

close all;
mep_flag = false;
save_fig_flag = false;
plot_flag = false;

config();
load('config.mat');

try
    load('baseline_struct.mat','baseline_struct');
catch
    baseline_struct = struct();
    b_index = 1;
end


% Initialise which directories to look at 
sel_dirs = {'.\Data\B_animals_sMEP_files\Intact', ...
            '.\Data\B_animals_sMEP_files\W01', ...
            '.\Data\B_animals_sMEP_files\W06'};

nmep_lim = 75;
nelec_ch = 5;
ch_names = {'Deltoid', 'Bicep', 'Pronator', 'Flexor', 'Extensor'};
Treatment = 'sham';

% Screen and figure setup
% get the computer screen size
scrsz = get(0,'ScreenSize');

MEP_win_time = (-PRE_WINDOW/fs : 1/fs : POST_WINDOW/fs)*fs'; 
MEP_pre_time = (-PRE3_MEP/fs:1/fs:POST3_MEP/fs)*1000';

% Loop through all the selected directories
for nsel = 1 : length(sel_dirs)
    sel_dir = cell2mat(sel_dirs(nsel));
    addpath(genpath(sel_dir));
    file_list = dir(sel_dir);
    flist = {file_list.name};
    flist = flist(contains(flist,'Group'));
    ngroups = length(flist);

    % If the selected directory contains 'intact' data then we perform a
    % baseline measurement protocol
    if contains(lower(sel_dir), 'intact')
        base_flag = true;
        base = struct();
    else
        base_flag = false;
    end
    % Initialise mep indexes for extraction of mep response
    index = 1;

    % Loop through however many groups there are in the selected directory
    for g = 1 : ngroups
        gname = cell2mat(flist(g));
        gnum  = str2num(gname(end));
        g_dir = [sel_dir '\' gname];

        % Create data and figure folders if they don't already exist
        mkdir([g_dir '\Figures']);
        mkdir([g_dir '\Data']);
        Save_filepath = [g_dir '\Figures\'];
        mep_filepath = [g_dir '\Data\'];

        % Get all files in group folder
        files = dir([g_dir '\*.bin']);

        %LOOP to process all files
        for n_file = 1:length(files)
            % Open a bin file
            chosenfile = files(n_file).name;
            titles = chosenfile(25:end-4);

            fileinfo = getFileInfo(chosenfile);
            Week = fileinfo.Week;
            AnNum = fileinfo.AnNum;
            Anode = fileinfo.Anode;
            Cathode = fileinfo.Cathode;
            Current = fileinfo.Current;

            nrows = 6; % no of row
            ncols = 1; % no of column
            % calculate subplot position by calling subfigrid funtion
            pos = subfigrid(nrows,ncols,...
              [0.07 0.01 0.02 0.13],[0.85 0.85]); % [left Right Top Bottom]

            %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            % Load EMG into data_array
            EMG_header = load_emg_header(chosenfile, 1);
            data_array = load_emg_data(chosenfile, ...
                EMG_header,EMG_header.data_position,1,inf,1);
            emg_data = data_array(:,1:nelec_ch);
            stim_pulse = data_array(:,6);

            %ZEROS
            % average MEPs
            mean_MEP = zeros(PRE_WINDOW+POST_WINDOW+1,nelec_ch);
            % rectified MEPs
            rec_MEP = zeros(PRE_WINDOW+POST_WINDOW+1,nelec_ch);
            % rectified MEPs for baseline
            rec_burst = zeros(PRE_WINDOW+1,nelec_ch);
            % response variables (peak to peak amplitude)
            early_amp = zeros(1,nelec_ch); 
            late_amp  = zeros(1,nelec_ch);
            % latency variables
            latency_early = zeros(1,nelec_ch);
            % area under rectified MEP responses
            auc_early = zeros(1,nelec_ch);
            auc_late = zeros(1, nelec_ch);

            % creating time vector
            inds = 1 : size(data_array,1);
            tend = length(data_array(:,STIM_CH))/fs;
            tvec = 0 : 1/fs : tend;
            tvec(1) = [];   % fs start from time zero 

            % Filter all data
            emg_filt = bpFilter(emg_data, 2, 30, 1e3, nq);
            [~, emg_tkeo] = energyop(emg_filt, false);
            emg_tkeo = reshape(emg_tkeo,size(emg_filt));
            emg_env = abs(lpFilterEMG(emg_tkeo, 2, 50, nq));

            % Select the type of data to be worked on. Either filt of
            % normalized to baseline. If this is not a baseline run, then we
            % select the normalized data as default.
            if ~base_flag
                emg_norm = normalizeEMGData(emg_filt, ...
                    baseline_struct, AnNum, Anode, Cathode, Current);
                emg_ac = emg_norm;
            else
                emg_ac = emg_filt;
            end

            % Perform thresholding (mu + J*sd)
            thold = thresholding(emg_env, THR_MUL);
            active_mask = emg_env > thold;

            % map all the stim pulses based on the activity
            new_stim = map_stimpulse(active_mask, stim_pulse);

            % Knowing that all stim pulses are equal to 0, match them across
            % channels
            for c = 1 : nelec_ch - 1
                if c == 1
                    final_stim = and(round(new_stim(:,c)), ...
                        round(new_stim(:,c+1)));
                else
                    final_stim = and(final_stim, ...
                        round(new_stim(:,c+1)) );
                end
            end

            % Get number of registered MEPs
            session = length(final_stim);

            %PLOT1
            if plot_flag
                h1 = figure(1);
                set(h1, 'name',titles,'Position', ...
                    [1 scrsz(2) scrsz(3) scrsz(4)]);

                for c = 1 : nelec_ch
                    % plot figure
                    % subplot('position',pos(c,:,1)); 
                    subplot(nelec_ch+1,1,c);
                    plot(tvec, emg_ac(:,c), 'k');
                    hold on;
                    plot(tvec, active_mask(:,c), 'c');
                    plot(tvec, emg_env(:,c)*5, 'g');
                    plot([tvec(1) tvec(end)], [thold(c) thold(c)]*5, 'b');
                    plot(tvec, new_stim(:,c), 'r', 'LineWidth', 1.5);
                    xlim([1 length(tvec)/fs]); ylim([-1 1]);
                    set(gca,'fontsize',16); 
                    ylabel(['Ch ' num2str(c)],'fontsize',16);
                end
                subplot('position',pos(6,:,1)); 
                plot(tvec, final_stim, 'b', 'LineWidth', 1.5);
                xlim([1 session/fs]); ylim([-0.5 1.5]);
                set(gca,'fontsize',16); 
                xlabel('Sec','fontsize',16); ylabel('Ch 6','fontsize',16);
                if save_fig_flag
                    saveas(h1, [Save_filepath, heading, '.tif']);
                end
            end

            % MEP responses here
            b_name = [AnNum '\'];
            dir_list = dir([mep_filepath b_name]);
            if length(dir_list) < 1
                mkdir([mep_filepath b_name]);
            end
            if mep_flag
                for c = 1 : 5
                    s1 = sprintf('ch%d',c);
                    csv_fname = sprintf('%s\\%s\\%s_%s', ...
                        mep_filepath,b_name,...
                        fname, s1);

                    [iemg_temp , pk2pk_temp] = mepresponse(...
                        filtdata(:,c), ...
                        new_stim(:,c), fs, c);

                    iemg(index, 1).Week = Week;
                    iemg(index, 1).AnNum = AnNum;
                    iemg(index, 1).Anode = Anode;
                    iemg(index, 1).Cathode = Cathode;
                    iemg(index, 1).Current = Current;
                    iemg(index, 1).Group = gnum;
                    iemg(index, 1).Channel = c;
                    iemg(index, 1).MiddleResponses = iemg_temp.mr;
                    iemg(index, 1).LateResponses = iemg_temp.lr;

                    pk2pk(index, 1).Week = Week;
                    pk2pk(index, 1).AnNum = AnNum;
                    pk2pk(index, 1).Anode = Anode;
                    pk2pk(index, 1).Cathode = Cathode;
                    pk2pk(index, 1).Current = Current;
                    pk2pk(index, 1).Group = gnum;
                    pk2pk(index, 1).Channel = c;
                    pk2pk(index, 1).MRMeans = pk2pk_temp.mr_means;
                    pk2pk(index, 1).LRMeans = pk2pk_temp.lr_means;
                    pk2pk(index, 1).MRPeak2Peak = pk2pk_temp.mr;
                    pk2pk(index, 1).LRPeak2Peak = pk2pk_temp.lr;
                    pk2pk(index, 1).MRNumPeaks = pk2pk_temp.mr_num_pks;
                    pk2pk(index, 1).LRNumPeaks = pk2pk_temp.lr_num_pks;
                    
                    index = index + 1;
                    
                end
            end

            % Get edges of mep stim pulses
            pulse_thold = SLOPE;
            pulse_offset = [final_stim(2:end); NaN];
            risingEdge = inds(final_stim < pulse_thold & ...
                pulse_offset > pulse_thold);

            counter = risingEdge';
            % Get MEP traces
            [meps, pk2pk, latency_early, iemg] = meptrace(emg_ac, counter);
            out_MEPs = meps.out_MEPs;
            mean_MEP = meps.mean_MEP;
            rec_MEP  = meps.rec_MEP;

            early_amp = pk2pk.early_pk2pk;
            late_amp  = pk2pk.late_pk2pk;
            auc_early = iemg.early;
            auc_late = iemg.late;
            if plot_flag
                h2 = figure(2);
                figure(h2, 'name', heading, ...
                    'Position',[1 scrsz(2) scrsz(3) scrsz(4)]);
                for c = 1 : nelec_ch
                    subplot('position',pos(1,:,c));
                    plot(MEP_win_time, mean_MEP1, 'k',...
                        'LineWidth', 2.5);
                    plot([0, 0], [ALL_ymin ALL_ymax], 'r')
                    ylim([ALL_ymin ALL_ymax]); 
                    xlim([-PRE_WINDOW POST_WINDOW]);
                    title(cell2mat(ch_names(c)), 'interpreter', ...
                        'none', 'fontsize',14); ylabel('mV','fontsize',16);

                    subplot('position',pos(2,:,c)); 
                    plot(MEP_win_time, rec_MEP1, 'b', 'LineWidth', 1.5);
                    hold on;

                    %adds line to mark stimulation pulse onset
                    plot([0, 0], [ALL_ymin ALL_ymax], 'r') 
                    hold off
                    ylim([0 AVG_ymax]); 
                    xlim([-PRE_WINDOW POST_WINDOW]);
                    title(cell2mat(ch_names(c)), 'interpreter', ...
                        'none', 'fontsize',14); 
                    xlabel('ms','fontsize',18); ylabel('|mV|','fontsize',16);
                end
            end

            if save_fig_flag
                saveas(h2, [Save_filepath, heading, '.tif']);
            end

            %=========================================================================
            %PLOT3
            scrsz = get(0,'ScreenSize');
            nrows = 1; % no of row
            ncols = 5; % no of column
            % calculate subplot position by calling subfigrid funtion
            pos = subfigrid(nrows,ncols,...
                  [0.06 0.01 0.06 0.08],[0.84 0.88]); % [left Right Top Bottom]

            % h2 = figure('name',heading,'Position',[1 scrsz(2) scrsz(3) scrsz(4)]);

            if base_flag
                %{
                    chk_len = size(out_MEP1,2);
                    if chk_len < nmep_lim
                        lim = chk_len;
                    else
                        lim = nmep_lim;
                    end
                %}
            % get min from all recordings across each channel and move to
            % 2d array
                out_mins = squeeze(min(out_MEPs, [], 1));
                out_maxs = squeeze(max(out_MEPs, [], 1));
                abs_maxs = squeeze(max(abs(out_MEPs),[],1));


                split_fname = split(chosenfile(1 : end-4),'_');
                week = split_fname{2};

                baseline_struct.(week)(b_index).AnNum = AnNum;
                baseline_struct.(week)(b_index).Group = gnum;
                baseline_struct.(week)(b_index).Anode = Anode;
                baseline_struct.(week)(b_index).Cathode = Cathode;
                baseline_struct.(week)(b_index).Current = Current;
                baseline_struct.(week)(b_index).maxs = out_maxs;
                baseline_struct.(week)(b_index).mins = out_mins;
                baseline_struct.(week)(b_index).abs_max = abs_maxs;

                b_index = b_index + 1;
            end

            %----------------------------------------------------------------------- 
            if size(out_MEPs,2) > nmep_lim
                plot_MEPs = out_MEPs(:, 1:nmep_lim, :);
            else
                plot_MEPs = out_MEPs;
            end
            offset = repmat(1:size(plot_MEPs,2), size(plot_MEPs,1), 1);

            for c = 1 : 5
                subplot('position',pos(1,:,c));
                plot(MEP_pre_time, plot_MEPs(:, :, c)+offset, 'k');
                hold on
                plot ([0,0], [19,61], 'r');
                plot ([10,10], [19,61], 'b');
                hold off
                ylim([19, 61]);
                xlim([-PRE_MEP POST_MEP]); 
                title(cell2mat(ch_names(c)), ...
                    'interpreter', 'none', 'fontsize',14);
            end

            close all;
            %{
                txt_output = {heading, AnNum, Week, Treatment, Current, Anode, Cathode};
                num_output = [trials, early_amp1, early_amp2, early_amp3,...
                    early_amp4, early_amp5, late_amp1, late_amp2, late_amp3, ...
                    late_amp4, late_amp5, latency_early1, latency_early2,...
                    latency_early3, latency_early4, latency_early5, auc_early1,...
                    auc_early2, auc_early3, auc_early4, auc_early5, auc_late1,...
                    auc_late2, auc_late3, auc_late4, auc_late5];

                if exist([Save_filepath, 'MEPvalues.xlsx'], 'file') == 0
                    chosenfile = [Save_filepath, 'MEPvalues.xlsx'];
                    sheet = 1;
                    x1cell = 'A2';
                    x2cell = 'H2';
                    labels ={'heading', 'AnNum', 'Week', 'Treatment', 'Current', 'Anode','Cathode','Trials','Deltoid early P-P','Biceps eP-P','Pronator eP-P','Flexor eP-P','Extensor eP-P','Deltoid late P-P','Biceps lP-P','Pronator lP-P','Flexor lP-P','Extensor 1P-P','D latency','B latency','P latency','F latency','E latency','D eAUC','B eAUC','P eAUC','F eAUC','E eAUC','D lAUC','B lAUC','P lAUC','F lAUC','E lAUC'};
                    xlswrite(chosenfile,labels);
                    [num,txt,raw]=xlsread(chosenfile);
                    xlswrite(chosenfile,txt_output,sheet, x1cell);
                    xlswrite(chosenfile,num_output,sheet, x2cell);
                    fprintf([heading '\n']);
                elseif exist([Save_filepath, 'MEPvalues.xlsx'],'file') > 0
                     chosenfile = [Save_filepath, 'MEPvalues.xlsx'];
                    x1cell = 'A2';
                    x2cell = 'H2';
                    [num,txt,raw]=xlsread(chosenfile);
                    xlswrite(chosenfile,vertcat(raw(2:end,1:7),txt_output),1,x1cell);
                    xlswrite(chosenfile,vertcat(num(1:end,6:end),num_output),1,x2cell);
                    fprintf([heading '\n']);
                end
            %}

            fclose('all');
            %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            clear out_MEPs;

            fprintf('%d/%d ... %s\n', n_file, length(files), chosenfile);
        end
    end
    save('baseline_struct.mat','baseline_struct','-v7.3');
    if mep_flag
        struct2csv(iemg , 'iemg.csv');
        struct2csv(pk2pk, 'pks.csv');
    end
end
    
end
