function emg_norm = normalizeEMGData(emg_data, ...
    baseline_struct, an, anode, cathode, current)
% This function normalizes emg_data against the baseline measured emg data from
% the data structure named 'baseline_struct'. Normalisation is specific to the
% configuration
% USAGE
% emg_norm = normalizeEMGData(emg_data, ...
%   baseline_struct, an, anode, cathode, current)
% emg_data = EMG recording
% baseline_struct = data structure of baseline measurements
% an = animal using 'B##' nomenclature
% anode = stimulating anode
% cathode = stimulating cathode
% current = stimulating current

% Initialise arrays
an_arr = [];
anode_arr = [];
cathode_arr = [];
current_arr = [];

% Concatenate all matching fields
an_arr = catStructField( baseline_struct.W00, 'AnNum' );
anode_arr = catStructField( baseline_struct.W00, 'Anode' );
cathode_arr = catStructField( baseline_struct.W00, 'Cathode' );
current_arr = catStructField( baseline_struct.W00, 'Current' );
current_arr = cellfun( @str2double, current_arr);

% Find the row index where all conditions are met
an_mask = strcmp(an_arr, an);
anode_mask = strcmp(anode_arr, anode);
cathode_mask = strcmp(cathode_arr, cathode);
curr_mask = current_arr == current;
[~, ind] = max( an_mask & anode_mask & cathode_mask & curr_mask );

% Set baseline value to be that given index and create normalised data
baseline = baseline_struct.W00(ind);
mu = mean(baseline.abs_max);
sd = std(baseline.abs_max);

norm_fact = mu + sd;
emg_norm = emg_data./norm_fact;

end

